from pathlib import Path
from sys import platform

from pytest import mark

from venvflon import utils


@mark.slow
@mark.skipif(condition=platform != 'win32', reason='Run only on Windows')
@mark.parametrize('cmd, result', [('Clear-Host', 0), ('bullshit', -1)])
def test_run_command(cmd, result):
    rc = utils.run_command(cmd=['powershell', cmd])
    assert rc == result


def test_success_deep_1_venv_list_in(resources):
    venvs = utils.venv_list_in(current_path=resources, max_depth=1)
    assert len(venvs) == 3
    assert sorted([venv.name for venv in venvs]) == ['.venv_310', '.venv_311', '.venv_312']


def test_success_deep_2_venv_list_in(resources):
    venvs = utils.venv_list_in(current_path=resources, max_depth=2)
    assert len(venvs) == 4
    assert sorted([venv.name for venv in venvs]) == ['.venv_310', '.venv_311', '.venv_312', '.venv_39']


def test_failure_venv_list_in():
    venvs = utils.venv_list_in(current_path=Path('..'), max_depth=1)
    assert len(venvs) == 0



from __future__ import annotations

from collections.abc import Sequence
from os import sep, walk
from pathlib import Path
from subprocess import CalledProcessError, run


def run_command(cmd: Sequence[str], cwd: Path | None = None) -> int:
    """
    Run command in shell as a subprocess.

    :param cmd: The command to be executed as a sequence of strings
    :param cwd: current working directory
    :return: The return code of command
    """
    try:
        proc = run(cmd, check=True, shell=False, cwd=cwd)
        return proc.returncode
    except CalledProcessError as e:
        print(f'Result: {e}')
        return -1

def get_command_output(cmd: Sequence[str], cwd: Path | None = None) -> tuple[int, str, str]:
    """
    Execute command and return its output.

    :param cmd: The command to be executed as a sequence of strings
    :param cwd: current working directory
    :return: Tuple with return code, stderr and stdout
    """
    try:
        result = run(cmd, capture_output=True, check=True, cwd=cwd)
        return result.returncode, result.stderr.decode('utf-8'), result.stdout.decode('utf-8')
    except CalledProcessError as e:
        print(f'Result: {e}')
        return e.returncode, e.stderr.decode('utf-8'), e.stdout.decode('utf-8')


def make_sym_link(to_path: Path, target: Path) -> None:
    """
    Make symbolic link.

    :param to_path: path to symbolic link
    :param target: target path
    """
    cmd_symlink = f'"New-Item -ItemType SymbolicLink -Path \\"{to_path}\\" -Target \\"{target}\\"'
    ps_command = f"Start-Process powershell.exe -ArgumentList '-Command {cmd_symlink}' -Verb RunAs"
    print(f'Make symbolic link: {ps_command}')
    run_command(cmd=['powershell.exe', '-Command', ps_command])


def rm_sym_link(sym_link: Path) -> None:
    """
    Remove symbolic link.

    :param sym_link: path to symbolic link
    """
    rm_symlink = f"(Get-Item '{sym_link}').Delete()"
    ps_command = f'Start-Process powershell.exe -ArgumentList "-Command {rm_symlink}" -Verb RunAs'
    print(f'Execute: {ps_command}')
    run_command(cmd=['powershell.exe', '-Command', ps_command])


def venv_list_in(current_path: Path, max_depth: int = 1) -> Sequence[Path]:
    """
    Find all virtual environments in given path.

    :param current_path: path to search in
    :param max_depth: maximum depth of search
    :return: list of paths to virtual environments
    """
    result = []
    for dirpath, dirnames, _ in walk(current_path, topdown=True):
        for dirname in dirnames:
            if '.venv_' in dirname:
                result.append(Path(dirpath) / dirname)
        if dirpath.count(sep) - str(current_path).count(sep) == max_depth - 1:
            del dirnames[:]
    return result



from __future__ import annotations

from os import environ
from pathlib import Path
from sys import base_prefix

from venvflon.flon import Gui

environ['TCL_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tcl8.6')
environ['TK_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tk8.6')
import tkinter as tk

__version__ = '0.1.0'

def run():
    """Run the main GUI."""
    root_tk = tk.Tk()
    width, height = 300, 150
    root_tk.title(f'venvflon - v{__version__}')
    root_tk.geometry(f'{width}x{height}')
    root_tk.iconphoto(False, tk.PhotoImage(file=Path(__file__).parent / 'img' / 'cannula_64.png'))
    gui = Gui(master=root_tk)
    gui.mainloop()


if __name__ == '__main__':
    run()



from __future__ import annotations

from os import environ, getcwd
from pathlib import Path
from sys import base_prefix, executable
from time import sleep

from venvflon.utils import get_command_output, make_sym_link, rm_sym_link, venv_list_in

environ['TCL_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tcl8.6')
environ['TK_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tk8.6')
import tkinter as tk

print(executable)

class Gui(tk.Frame):
    """Tkinter GUI for venvflon."""

    def __init__(self, master: tk.Tk) -> None:
        """
        Tkinter  GUI for venvflon.

        :param master: Tkinter root
        """
        super().__init__(master)
        self.master = master
        self.venv = tk.StringVar(value=' ')
        self.status_txt = tk.StringVar()
        self.cwd_entry = tk.StringVar()
        self.cwd = Path(getcwd())
        self.cwd_entry.set(str(self.cwd))
        self.venv_list = venv_list_in(current_path=self.cwd)
        new_width, new_height = len(str(self.venv_list[0])) + 300, len(self.venv_list) * 55
        self.master.geometry(f'{new_width}x{new_height}')
        self.master.minsize(width=new_width, height=new_height)
        self.init_widgets()
        self.update_status()

    def init_widgets(self) -> None:
        """Initialize widgets."""
        self.master.columnconfigure(index=0, weight=1)
        cwd_label = tk.Label(self.master, text='cwd:')
        cwd_label.grid(row=0, column=0, sticky=tk.W)
        cwd = tk.Entry(master=self.master, textvariable=self.cwd_entry, width=len(str(self.venv_list[0])) + 2)
        cwd.grid(row=0, column=1, sticky=tk.W)
        cwd.bind('<Return>', self.refresh_cwd)
        self.add_venvs()

    def add_venvs(self):
        """Add venvs as radio buttons to the GUI."""
        venv_label = tk.Label(self.master, text='venv:')
        venv_label.grid(row=1, column=0, sticky=tk.W)
        frame = tk.Frame(master=self.master, relief=tk.GROOVE, borderwidth=2)
        frame.grid(row=1, column=1, columnspan=2, padx=2, pady=2, rowspan=len(self.venv_list))
        for i, text in enumerate(self.venv_list, 1):
            rb_venvs = tk.Radiobutton(master=frame, text=str(text), variable=self.venv, value=text, command=self.venv_selected)
            rb_venvs.grid(row=i, column=1, pady=0, padx=2, sticky=tk.W)
        status = tk.Label(master=self.master, textvariable=self.status_txt)
        status.grid(row=len(self.venv_list) + 1, column=0, columnspan=3, sticky=tk.E)

    def refresh_cwd(self, *args):
        """
        Refresh the current working directory.

        :param args: internal tkinter arguments
        """
        self.venv_list = venv_list_in(current_path=self.cwd)
        self.add_venvs()

    def venv_selected(self):
        """Set the selected venv as the active one."""
        new_venv = self.venv.get()
        rm_sym_link(sym_link=Path(getcwd()) / '.venv')
        make_sym_link(to_path=Path(getcwd()) / '.venv', target=Path(new_venv))
        sleep(0.8)
        self.update_status()

    def update_status(self):
        """Update the status text."""
        out = get_command_output(cmd=['python', '-V'])
        self.status_txt.set(f'Current: {out[2].strip()}')
